#!/usr/bin/env perl
#############################################################################
#  suppliers.pl - convert MDL ACD supplier file to HTML.  Based on
#                 Dave's C version.
#
#  State
#      0   looking for something
#      1   coden
#      2   company
#      3   catalog
#      4   name & address
#      5   other suppliers
#      6   tel/fax/telex/cable/internet/email (from 4 or 5)
#      7   tel/fax/telex/cable/internet/email (from 6 or 7)
#      8   compounds in database 
#      9   type of compounds 
#      10   file notes
#
# Author: Jeremy Yang
# 12 Jul 1999
#############################################################################

$nin = 0;

while ($buf = <STDIN>) {

   chop($buf);

   ##################################################################
   ### DTYPE 
   ##################################################################
   if ($buf =~ /^\$DTYPE /) {

      #########################################################
      ### MOL:MOLNAME starts things rolling. 
      #########################################################
      if ($buf =~ /^\$DTYPE MOL:MOLNAME/) {
        if ($ok) { $armed = $state = 1; }
        $ok++;

      #########################################################
      ### Else need armed SUPPLIER data. 
      #########################################################
      } elsif (!$armed || ($buf !~ /^.......MOL:SUPPLIER\(/)) {
        $state = 0;
      }
      next;
   }

   ##################################################################
   ### Else need to be armed. 
   ##################################################################
   if (!$armed) { $state = 0; next; }

   ##################################################################
   ### Need DATUM from here on, get value. 
   ##################################################################
   next if ($buf !~ /^\$DATUM /);
   ($_ = substr($buf, 7)) =~ s/^\s*//;

   ##################################################################
   ### Armed state 0: Pick a few. 
   ##################################################################
   if (0 == $state) {
      if (/^Company name:/) {
           $state = 2;
      } elsif (/^Catalog:/) {
           $state = 3;  $catrep = 0;

      } elsif (/^Name and address of supplier:/) {
           $state = 4;  $addrep = 0;
      } elsif ($addrep &&  /upplier/ && /:/) {
           $state = 5;  $addrep = 0;

      } elsif (/^Approximate number of/) {
           $state = 8;
      } elsif (/^Type of Compounds:/) {
           $state = 9;  $notrep = 0;
      } elsif (/^File Notes:/) {
           $state = 10;  $notrep = 0;
      } else {
           $state = 0;
      }
      next;
   }

   ##################################################################
   ### State 1: DATUM is "Reserved..." or coden. 
   ### Coden is standardized company name for lookup.
   ##################################################################
   if (1 == $state) {
      if (/^RESERVED/) { $state = 0; next; }
      if ($_) {
         ($coden = $_) =~ s/[^A-Za-z0-9]//g;
         $coden =~ tr/[a-z]/[A-Z]/;
         print "\n<HR><A NAME=$coden><H2>$_</H2></A>\n";
         $nin++;
      }
      $state = 0;

   ##################################################################
   ### State 2: Company name. 
   ##################################################################
   } elsif (2 == $state) {
      if (/^\s*\.\s*$/) { print "<EM>Company name:</EM>&nbsp; $_<BR>\n"; }
      $state = 0;

   ##################################################################
   ### State 3: Catalog name. 
   ##################################################################
   } elsif (3 == $state) {
      if (/^\s*\.\s*$/) { print "<BR>\n"; $state = 0; next; }

      if ($catrep) { print " $_\n"; }
      else         { print "<EM>Catalog name:</EM>&nbsp; $_\n"; }
      $catrep++;

   ##################################################################
   ### State 4: Name and address. 
   ##################################################################
   } elsif (4 == $state) {

      if (/^\s*\.\s*$/) { print "\n"; $state = 6; next; }

      if ($addrep) { print ",\n$_"; }
      else         { print "<P><B>$_</B>"; }
      $addrep++;
      $notrep = 0;

   ##################################################################
   ### State 5: Other suppliers. 
   ##################################################################
   } elsif (5 == $state) {

      if (/^\s*\.\s*$/) { print "\n"; $state = 6; next; }

      if (/upplier/ && /:/) { next; }

      if ($addrep) { print ",\n$_"; }
      else         { print "<P><B>$_</B>"; }
      $addrep++;
      $notrep = 0;

   ##################################################################
   ### State 6/7: Telephone/Fax/Telex/Cable/Internet/Email. 
   ##################################################################
   } elsif (6 == $state || 7 == $state) {

     if (7 == $state && /^\s*\.\s*$/) {
        print "\n"; $state = 0; next;
      }

more7:
      if (/^Telephone: /) {
           printf(",\n&nbsp; <EM>phone</EM> %s", substr($_, 11));
      } elsif (/^Fax: /) {
           printf(",\n&nbsp; <EM>fax</EM> %s", substr($_, 5));
      } elsif (/^Telex: /) {
           printf(",\n&nbsp; <EM>telex</EM> %s", substr($_, 7));
      } elsif (/^Cable: /) {
           printf(",\n&nbsp; <EM>cable</EM> %s", substr($_, 7));
      } elsif (/^Internet: /) {
        ($url = substr($_, 10)) =~ s/\s*([^\s]+)\s*/$1/;

        if (length($url) == 22) { ## KLUDGE: may need another line
          <STDIN>; chop($buf = <STDIN>);
          if ($buf =~ /^\$DATUM                /) {
            ($_ = substr($buf, 7)) =~ s/^\s*//;
            $url .= $_;
          } elsif ($buf =~ /^\$DATUM\s*\.\s*$/) {
            print "\n"; $state = 0; next;
          } else {
            goto more7;
          }
        }

        print ",\n&nbsp; <EM>internet</EM> <A HREF=\"$url\">$url</A>";
      } elsif (/^Email: /) {
       ($email = substr($_, 6)) =~ s/\s*([^\s]+)\s*/$1/;

        if (length($email) == 22) { ## KLUDGE: may need another line
          <STDIN>; chop($buf = <STDIN>);
          if ($buf =~ /^\$DATUM                /) {
            ($_ = substr($buf, 7)) =~ s/^\s*//;
            $email .= $_;
          } elsif ($buf =~ /^\$DATUM\s*\.\s*$/) {
            print "\n"; $state = 0; next;
          } else {
            goto more7;
          }
        }

       print ",\n&nbsp; <EM>email</EM> <A HREF=\"mailto:$email\">$email</A>";
      } 

      $notrep++;
      $state = 7;

   ##################################################################
   ### State 8: Compounds in database. 
   ##################################################################
   } elsif (8 == $state) {
        if (/^\s*\.\s*$/) {
           $state = 0;
      } elsif (/^compounds in database: /) {
           printf("<P><EM>Compounds in database:</EM>&nbsp; %s\n",
       	   substr($_, 23));
           $state = 0;
        }

   ##################################################################
   ### State 9: Type of compounds 
   ##################################################################
   } elsif (9 == $state) {
        if (/^\s*\.\s*$/) { print "\n"; $state = 0; next; }

        if ($notrep) { print " $_\n"; }
        else       { print "<P><EM>Type of Compounds:</EM>&nbsp; $_\n"; }
        $notrep++;

   ##################################################################
   ### State 10: File notes. 
   ##################################################################
   } elsif (10 == $state) {
        if (/^\s*\.\s*$/) { print "\n"; $state = 0; next; }

        if ($notrep) { print " $_\n"; }
        else        { print "<P><EM>Notes:</EM>&nbsp; $_\n"; }
        $notrep++;
   }
}
print "\n";
print STDERR "ACD supplier records converted: $nin\n";
