#!/usr/bin/env perl
#
# This program converts non-structural data in an RDF file into the
# corresponding tdts. It reads from STDIN and writes to STDOUT while using
# conversion definitions from a file given as its parameter.
#
# usage: rdf2datatdt conversion-file <input.rdf >output.tdt
#

# Read conversion file
open (CONVERSION, $ARGV[0]) || die "Couldn't open " . $ARGV[0] .": $!\n";
$_ = <CONVERSION>;
($mdlextreg, $tdtextreg) = split(/\s/);
while (<CONVERSION>)
{
   if (/^#/) {next;}	# skip comments

   ($rdftag, $tdttag) = split(/\s/);
   $star = "";
   ($rdftag, $star) = split(/[()]/, $rdftag);
   ($tdttag, $tdtfields) = split(/[<>]/, $tdttag);

   $multiple{$rdftag} = $star;
   $rdf2tdttag{$rdftag} = $tdttag;
   $rdf2tdtfields{$rdftag} = $tdtfields;

   if ($tdtfields)
   {
      $i = 0;
      foreach $fld (split(/;/,$tdtfields))
      {
	 $i++;
	 $field_index2name{$tdttag."|".$i} = $fld;
	 $field_name2index{$tdttag."|".$fld} = $i;
      }
      $nfields{$tdttag} = $i;
   }
}
close (CONVERSION);

# now we scan the input for VARIATION sections to collect and
# transform the data.
while ($_=<STDIN>)
{
   if (/\$RFMT/)
   {
      ($d1, $d2, $intreg) = split(/\s/);
   }

   if (!/\$DTYPE/) {next;}

   $variation=0;
   # now we are at the data
   for (;;)
   {
      if (/\$RFMT/)
      {
	 ($d1, $d2, $intreg) = split(/\s/);
	 last;
      }
      if (!/^\$DTYPE/)
      {
	 if (!($_=<STDIN>)) {last;}	# skip to data part
	 next;
      }
      chop;	# get rid of trailing '\n'

      # we expect a line like '$DTYPE RXN:VARIATION(1):LITTEXT(1)' here
      # get datatype
      ($d1, $d2, $d3, $varno, $rest) = split(/[ ():][ ():]*/, $_, 5);
      # print "var=$varno, rest=$rest";
      if ($varno > $variation)	# print old subtdt if one was pending
      {
	 if ($variation > 0)
	 {
	    &printSubTDT();
	    &clearSubTDT();
	 }
	 $variation = $varno;
      }

      if (!($_=<STDIN>)) {last;}	# skip to data part

      # we should have a line like '$DATUM EBERLE and NUNINGER F' here
      # we could also have the connection table of some solvent or catalyst.
      if ($rest eq $mdlextreg)	# set external registry number
      {
	 ($d1, $extreg) = split(/\s\s*/);
	 if (!($_=<STDIN>)) {last;}
      }
      else			# process normal fields
      {
	 ($rdftag, $rest) = split(/[():]/, $rest, 2);
	 if (!$rdf2tdttag{$rdftag})	# this rdftag shall not be converted
	 {				#  => skip it
	    while (<STDIN>)
	    {
	       if (/^\$DTYPE/) {last;}
	    }
	    next;
	 }
	 elsif ($multiple{$rdftag})
	 {
	    ($row, $rest) = split(/[():][():]*/, $rest, 2);
	    $row =~ s/\s*$//;
	    $rest =~ s/\s*$//;
	 }
	 else
	 {
	    $row = "";
	 }

	 # Now we collect the contents of the data field
	 if (/\$MFMT/)		# connection table field
	 {			#  => collect multiple lines
	    $fld = "";
	    while (<STDIN>)
	    {
	       if (/^\$/) {last;}
	       $fld .= $_;
	    }
	 }
	 else
	 {			# only a single line
	    chop;	# get rid of '\n'
	    ($d, $fld) = split(/\s/, $_, 2);
	    $fld =~ s/^ *//; $fld =~ s/ *$//;	# trim away blanks
	 }

	 # now we set the appropriate entry in the associative arrays
	 $tdttag = $rdf2tdttag{$rdftag};
	 $datatags{"$tdttag|$row"} = 1;
	 $dataitems{"$tdttag|$row|" .
		    $field_name2index{$tdttag."|".$rest}} = $fld;
      }
   }
   # print pending tdt
   if ($variation >= 1)
   {
      &printSubTDT();
      &clearSubTDT();
   }
}

sub clearSubTDT
{
   undef $extreg;
   undef %datatags;
   undef %dataitems;
}

sub dy_quote
{
  local($input) = @_;
  if ($input =~ /[;<>"\$]/)      # special characters => need to quote
  {
    if ($input =~ /"/)          # internal quotes must be doubled
    {
      $input =~ s/"/""/g;
    }
    $input = "\"".$input."\"";
  }
  $input;
}

sub printSubTDT
{
   if ($extreg)
   {
      print "$tdtextreg<$extreg>\n";
   }
   else
   {
      print "\$RID_INTERNAL<$intreg"."_"."$variation>\n";
   }
   for $composed_tag (keys %datatags)
   {

      ($tag, $row) = split(/\|/, $composed_tag);
      if ($nfields{$tag}>0)	# possibly multiple row
      {
	 print "$tag<";
	 for $i (1..$nfields{$tag})
	 {
	    if ($i != 1) {print ";";}
	    $fld = $dataitems{$tag."|".$row."|".$i};
	    $fld = &dy_quote($fld);
	    print $fld;
	 }
	 print ">\n";
      }
      else		# only a single row
      {
	 $fld =  $dataitems{$composed_tag."|"};
	 $fld = &dy_quote($fld);
	 print "$tag<" . $fld . ">\n";
      }
   }
   print "|\n";
}

