#! /usr/local/bin/perl
#############################################################################
### wdi.cgi -- interface to WDI-Oracle/DayCart
#############################################################################
### Author: Jeremy Yang
### Date:     8 Oct 2001
#############################################################################
$| = 1;

require DU_JAVA;
require CGIPARSE;

#############################################################################
### Establish environment and check for auxiliary scripts & programs.
#############################################################################
($HOSTNAME = `/usr/ucb/hostname`) || ($HOSTNAME = `/usr/bin/hostname`) ||
($HOSTNAME = `/usr/bsd/hostname`);

$URL = $ENV{'SCRIPT_NAME'};
chop($PROG = `basename $0`);
chop($DATE = `date +'%Y%b%d%H%M'`);
$TMPFILE = "/tmp/${PROG}.${DATE}";

$ENV{'ORACLE_HOME'} = "/oracle/o815";
$SQLPLUS = "$ENV{'ORACLE_HOME'}/bin/sqlplus";

foreach ("./CGIPARSE.pm",$SQLPLUS)
  { (-f $_) || (push @ERRORS,"ERROR: Can't find \"$_\"..."); }

$ORUSR = "mug";     ### default Oracle user
$ORPWD = "coffee";  ### default Oracle pwd
$ORSVR = "aussie";  ### default Oracle server

$SIMMIN = "0.8";    ### default minimum similarity
$COLWIDTH = 20;     ### default column width
$MAXLINES = 1000;   ### default max output lines
$ALPHA = "1.0";     ### default Tversky alpha
$BETA = "0.0";      ### default Tversky beta
$DEBUG = "1";       ### default DayCart debug level (0,1,5,9)

##################################################################
### Hash of all dtypes w/ their full col.tab specs and descriptions
### for column headings.
##################################################################
%DTYPES = (
  SMILES => { col=>'SMILES', head=>'SMILES' },
  DXRN => { col=>'WDI_MAIN.DXRN', head=>'DXRN' },
  ISM_SMI => { col=>'ISM_SMI', head=>'Isomer' },
  SUBSTRUCTURES => { col=>'SUBSTRUCTURES', head=>'SubstrKeywds' },
  PREF_NAME => { col=>'PREF_NAME', head=>'DerwentPrefName' },
  ACTIVITY => { col=>'ACTIVITY', head=>'ActivityKeywds' },
  AVG_MOLWT => { col=>'AVG_MOLWT', head=>'AvgMolwt' },
  XY_COORDS => { col=>'XY_COORDS', head=>'XYCoords' },
  XY_COORDS_COMMENT => { col=>'XY_COORDS_COMMENT', head=>'XYCoordsComment' },
  MOLFORM => { col=>'MOLFORM', head=>'DerwentMolform' },
  UPDATE_CODE => { col=>'UPDATE_CODE', head=>'DerwentUpdateCode' },
  MECHANISM => { col=>'MECHANISM', head=>'Mechanism' },
  JOURNAL_REF => { col=>'JOURNAL_REF', head=>'JournalRef' },
  JOURNAL_PUB => { col=>'JOURNAL_PUB', head=>'Journal' },
  JOURNAL_ID => { col=>'JOURNAL_ID', head=>'JournalID' },
  SUBSTANCE_DESC => { col=>'SUBSTANCE_DESC', head=>'SubstDesc' },
  ADVERSE_EFFECTS => { col=>'ADVERSE_EFFECTS', head=>'AdverseEffects' },
  PRECAUTIONS => { col=>'PRECAUTIONS', head=>'Warnings' },
  INTERACTIONS => { col=>'INTERACTIONS', head=>'Interactions' },
  CONTRAINDICATIONS => { col=>'CONTRAINDICATIONS', head=>'Contraindications' },
  INDICATIONS_USAGE => { col=>'INDICATIONS_USAGE', head=>'IndicationsUsage' },
  REFERENCE => { col=>'REFERENCE', head=>'MiscRef' },
  CONFERENCE => { col=>'CONFERENCE', head=>'Conference' },
  CONFERENCE_REFNO => { col=>'CONFERENCE_REFNO', head=>'ConfFastTrackRef' },
  CONFERENCE_DETAILS => { col=>'CONFERENCE_DETAILS', head=>'ConfPresentation' },
  CONFERENCE_WDA_REFNO => { col=>'CONFERENCE_WDA_REFNO', head=>'ConfWDARef' },
  CAS => { col=>'WDI_CAS.CAS', head=>'CAS' },
  DERWENT_NAME => { col=>'WDI_DRN.DERWENT_NAME', head=>'DerwentName' },
  OTHERNAME => { col=>'WDI_NAM.NAM', head=>'OtherName' },
  APPR_NAME => { col=>'WDI_APN.APPR_NAME', head=>'ApprovedName' },
  APPR_NAME_CODE => { col=>'WDI_APN.APPR_NAME_CODE', head=>'ApprNameCode' },
  INTL_NAME => { col=>'WDI_INN.INTL_NAME', head=>'IntlNonProprietaryName' },
  INTL_NAME_STATUS => { col=>'WDI_INN.INTL_NAME_STATUS', head=>'INN_Status' },
  TRADENAME => { col=>'WDI_TN.TRADENAME', head=>'TradeName' },
  TRADENAME_MANUF => { col=>'WDI_TN.TRADENAME_MANUF', head=>'TradeNameManuf' },
  TRADENAME_COUNTRY => { col=>'WDI_TN.TRADENAME_COUNTRY', head=>'TradeNameCountry' },
  COMB_PREP => {col=>'WDI_COMB_PREP.COMB_PREP',head=>'CombPrep'},
  COMB_PREP_MANUF => {col=>'WDI_COMB_PREP.COMB_PREP_MANUF',head=>'CombPrepManuf'},
  COMB_PREP_COUNTRY => {col=>'WDI_COMB_PREP.COMB_PREP_COUNTRY',head=>'CombPrepCountry'},
  COMB_PREP_COMPONENTS => {col=>'WDI_COMB_PREP.COMB_PREP_COMPONENTS',head=>'CombPrepComponents'},
  US_ADOPTED_NAME => { col=>'WDI_USAN.US_ADOPTED_NAME', head=>'US_AdoptedName' },
  US_ADOPTED_NAME_CODE => { col=>'WDI_USAN.US_ADOPTED_NAME_CODE', head=>'USAN-Code' }
  );

##################################################################
### All Oracle tables.   
##################################################################
@TABLES = ('WDI_MAIN','WDI_APN','WDI_CAS','WDI_COMB_PREP','WDI_DRN',
  'WDI_NAM','WDI_TN','WDI_USAN','WDI_INN','WDI_DBINFO');

@NAMECOLS = ('DXRN','APPR_NAME','TRADENAME','DERWENT_NAME','INTL_NAME','NAM',
'PREF_NAME');

&PARSE_FORM_DATA(*FORM);  ## *FORM is "type glob"

##################################################################
### HTML header
##################################################################
print <<__EOF__;
Content-type: text/html

<HTML><HEAD><TITLE>$PROG</TITLE></HEAD>
<SCRIPT LANGUAGE="JavaScript">
function openWindow(url,name) {
  popupWin = window.open(url,name,"width=600,height=400,scrollbars=1,resizable=1,left=20,top=20,toolbar=0,location=0");
}
</SCRIPT>

<BODY BGCOLOR="#BBBBBB">
__EOF__


if ($FORM{'query'}) {
  &PRINTFORM();
  &QUERY();
  &PRINTFOOTER();
} elsif ($FORM{'help'}) {
  &HELP();
  &PRINTFOOTER();
} elsif ($FORM{'clear'}) {
  %FORM = ();
  &PRINTFORM();
  &PRINTFOOTER();
} elsif ($FORM{'describe'}) {
  &PRINTFORM();
  &DESCRIBE;
  &PRINTFOOTER();
} else {
  &PRINTFORM();
  &PING;
  &PRINTFOOTER();
}

system("rm ${TMPFILE}.sql") if (-f "${TMPFILE}.sql");

exit (0);


#############################################################################
### PRINTFORM - 
#############################################################################
sub PRINTFORM
{
  ############################################
  ### Handle context
  ############################################
  if ($FORM{'hideform'}) { return; }
  if ($FORM{'showsql'}) { $SHOWSQL = "CHECKED"; }
  if ($FORM{'showdup'}) { $SHOWDUP = "CHECKED"; }
  if ($FORM{'nhitsonly'}) { $NHITSONLY = "CHECKED"; }
  if ($FORM{'casesen'}) { $CASESEN = "CHECKED"; }
  if ($FORM{'ism'}) { $ISM = "CHECKED"; }
  if (!$FORM{'user'}) { $FORM{'user'} = $ORUSR; }
  if (!$FORM{'pwd'}) { $FORM{'pwd'} = $ORPWD; }
  if (!$FORM{'svr'}) { $FORM{'svr'} = $ORSVR; }
  if (!$FORM{'simmin'}) { $FORM{'simmin'} = $SIMMIN; }
  elsif ($FORM{'simmin'} < 0.6) {
    $FORM{'simmin'} = 0.6;
    push @ERRORS,"WARNING: minimum similarity allowed is 0.6.";
  }
  if (!$FORM{'colwidth'}) { $FORM{'colwidth'} = $COLWIDTH; }
  if (!$FORM{'maxlines'}) { $FORM{'maxlines'} = $MAXLINES; }
  if (!$FORM{'alpha'}) { $FORM{'alpha'} = $ALPHA; }
  if (!$FORM{'beta'}) { $FORM{'beta'} = $BETA; }
  if (!$FORM{'debug'}) { $FORM{'debug'} = $DEBUG; }

  if ($FORM{'qtype'} eq "exact") { $EXACT = "CHECKED"; }
  elsif ($FORM{'qtype'} eq "sim") { $SIM = "CHECKED"; }
  elsif ($FORM{'qtype'} eq "tver") { $TVER = "CHECKED"; }
  elsif ($FORM{'qtype'} eq "sub") { $SUB = "CHECKED"; }
  elsif ($FORM{'qtype'} eq "sma") { $SMA = "CHECKED"; }

  if ($FORM{'qtype'} eq "xnam") { $XNAM = "CHECKED"; }
  elsif ($FORM{'qtype'} eq "subnam") { $SUBNAM = "CHECKED"; }

  print <<__EOF__;
<CENTER><H2>$PROG</H2></CENTER>
<FORM NAME="${PROG}Form" METHOD=POST ACTION="$URL">
__EOF__

  &PRINT_JAVAGRINSSCRIPT("${PROG}Form","insmi");

print <<__EOF__;
<HR>
<TABLE WIDTH=100% CELLPADDING=5>
  <TR BGCOLOR="#DDDDDD"><TD COLSPAN=2>
    <B>Server:</B>
    <INPUT TYPE=TEXT NAME="svr" VALUE="$FORM{'svr'}" SIZE=10>
    <B>User:</B>
    <INPUT TYPE=TEXT NAME="user" VALUE="$FORM{'user'}" SIZE=8>
    <B>Pass:</B>
    <INPUT TYPE=PASSWORD NAME="pwd" VALUE="$FORM{'pwd'}" SIZE=8> <BR>
  </TD></TR>

  <TR BGCOLOR="#DDDDDD"><TD>
    <CENTER><B>Query:</B></CENTER><BR>
    <B>SMILES/SMARTS:</B>
    &nbsp;&nbsp;<INPUT TYPE=CHECKBOX NAME="ism" VALUE="TRUE" $ISM> 
    Isomeric<BR>
    <INPUT TYPE=text NAME="insmi" VALUE="$FORM{'insmi'}" SIZE=35>
    <INPUT TYPE=BUTTON VALUE="JavaGRINS" onClick="makeNewWindow()"> <BR>
    &nbsp;&nbsp;<INPUT TYPE="RADIO" NAME="qtype" VALUE="exact" $EXACT>
      Exact structure match<BR>
    &nbsp;&nbsp;<INPUT TYPE="RADIO" NAME="qtype" VALUE="sim" $SIM>
      Similar structures&nbsp;
    <INPUT TYPE=TEXT NAME="simmin" VALUE="$FORM{'simmin'}" SIZE=3>
    &nbsp;<INPUT TYPE="RADIO" NAME="qtype" VALUE="tver" $TVER>
      Tversky&nbsp;
    A:<INPUT TYPE=TEXT NAME="alpha" VALUE="$FORM{'alpha'}" SIZE=3>&nbsp;
    B:<INPUT TYPE=TEXT NAME="beta" VALUE="$FORM{'beta'}" SIZE=3><BR>
    &nbsp;&nbsp;<INPUT TYPE="RADIO" NAME="qtype" VALUE="sub" $SUB>
      Substructure <BR>
    &nbsp;&nbsp;<INPUT TYPE="RADIO" NAME="qtype" VALUE="sma" $SMA>
      SMARTS match <BR>
    <B>NAME:</B><INPUT TYPE=text NAME="name" VALUE="$FORM{'name'}" SIZE=16>
    <INPUT TYPE="RADIO" NAME="qtype" VALUE="xnam" $XNAM>Exact
    <INPUT TYPE="RADIO" NAME="qtype" VALUE="subnam" $SUBNAM>Substring&nbsp;
    <INPUT TYPE=CHECKBOX NAME="casesen" VALUE="TRUE" $CASESEN>
    Case sensitive<BR>
    <B>DXRN:</B>
    <INPUT TYPE=text NAME="dxrn" VALUE="$FORM{'dxrn'}" SIZE=12><BR>
    <B>CAS:</B>
    <INPUT TYPE=text NAME="cas" VALUE="$FORM{'cas'}" SIZE=12><BR>

  </TD><TD VALIGN=TOP>
    <CENTER><B>Output options:</B></CENTER><BR>
    <B>Datatypes:</B><BR>
<SELECT SIZE=8 MULTIPLE NAME="dtypes">
__EOF__

  ###########################################################################
  ### This cleverness needed to match, for example, "TRADENAME" but not
  ### "TRADENAME_MANUF".
  ###########################################################################
  $_ = "\000".$FORM{'dtypes'}."\000";
  foreach $dtype (sort keys(%DTYPES)) {
    printf "<OPTION %s>$dtype",(/\000${dtype}\000/)?"SELECTED":"";
  }

  print <<__EOF__;
    </SELECT><BR>
    Show SQL:
    <INPUT TYPE=CHECKBOX NAME="showsql" VALUE="TRUE" $SHOWSQL>&nbsp;
    Hide form:
    <INPUT TYPE=CHECKBOX NAME="hideform" VALUE="TRUE">&nbsp;
    Show duplicates:
    <INPUT TYPE=CHECKBOX NAME="showdup" VALUE="TRUE" $SHOWDUP>&nbsp;
    Show # of hits only:
    <INPUT TYPE=CHECKBOX NAME="nhitsonly" VALUE="TRUE" $NHITSONLY>&nbsp;

  <BR>
    Col width: 
    <INPUT TYPE=text NAME="colwidth" VALUE="$FORM{'colwidth'}" SIZE=3>&nbsp;
    Max output lines: 
    <INPUT TYPE=text NAME="maxlines" VALUE="$FORM{'maxlines'}" SIZE=4><BR>
    DayCart debug level: 
    <INPUT TYPE=text NAME="debug" VALUE="$FORM{'debug'}" SIZE=2><I>0,1,5,9</I><BR>
  </TD></TR>
__EOF__

  if ($FORM{'showsql'}) {
    print <<__EOF__;
  <TR BGCOLOR="#DDDDDD"><TD COLSPAN=2>
    <CENTER>
    SQL: <INPUT TYPE=text NAME="sql" VALUE="$FORM{'sql'}" SIZE=70>
    </CENTER></TD></TR>
__EOF__
  }

print <<__EOF__;
  <TR BGCOLOR="#DDDDDD"><TD COLSPAN=2>
    <CENTER>
    <B><INPUT TYPE=SUBMIT NAME="query" VALUE=" Submit Query "></B>&nbsp;
    <B><INPUT TYPE=SUBMIT NAME="describe" VALUE=" Describe Tables "></B>&nbsp;
    <B><INPUT TYPE=SUBMIT NAME="clear" VALUE=" Clear "></B>&nbsp;
    <B><INPUT TYPE=BUTTON VALUE="Help" onClick="openWindow('$URL?help=TRUE','subwin')"></B>
    </CENTER>
  </TD></TR>
</TABLE>
</FORM>
__EOF__
}

#############################################################################
### PRINTFOOTER - print errors, notes, and footer.
#############################################################################
sub PRINTFOOTER
{
  if ($#ERRORS>=0) { print "<HR>\n".join("<BR>", @ERRORS); }
  print <<__EOF__;
<HR>
<IMG SRC="/dayhtml/icons/sunburst-94x42.gif">
<A HREF="http://www.daylight.com">
Daylight Chemical Information Systems Inc.</A>
<BR><EM>support\@daylight.com</EM>
</BODY></HTML>
__EOF__
}


#############################################################################
### QUERY - Submit Oracle/DayCart query via SQLPlus
#############################################################################
sub QUERY
{
  my $sql;
  my @fulldtypes;
  my $tables;
  my %tables;

  if (!$FORM{'dtypes'}) {
    $FORM{'dtypes'} = "DXRN";
    push @ERRORS,"WARNING: No datatypes selected.  DXRN autoselected.";
  }

  open(TMP,">${TMPFILE}.sql");
  if ($FORM{'debug'} != 1) {
    $sql .= "SELECT c\$dcischem.ddpackage.fsetdebug($FORM{'debug'}) FROM DUAL;\n";
  }
  $sql .= "SET PAGESIZE 100;\n";

  #########################################################
  ### Establish columns (datatypes) selected, their tables
  ### and headings.
  #########################################################
  foreach (split(/\000/,$FORM{'dtypes'})) {
    push @fulldtypes,$DTYPES{$_}->{col};
    $sql .= sprintf "COLUMN %s HEADING %s FORMAT A%d;\n",
      $_,$DTYPES{$_}->{head},$FORM{'colwidth'};
  }
  $dtypes = join(",",@fulldtypes);
  $tables{'WDI_MAIN'} = 1;
  foreach (@fulldtypes) {
    if (/^(.*)\.(.*)$/) { $tables{$1} = 1; }
  }
  if ($FORM{'qtype'} eq "cas") { $tables{'WDI_CAS'} = 1; }
  $tables = join(",",keys(%tables));

  print "<HR><TABLE WIDTH=100% CELLPADDING=5><TR><TD BGCOLOR=WHITE>\n";

  $select = $FORM{'showdup'} ? "SELECT" : "SELECT DISTINCT";

  #########################################################
  ### SMILES/SMARTS query
  #########################################################
  if ($FORM{'insmi'}) {
    $smi = sprintf "smi2cansmi('%s',%d)",$FORM{'insmi'},($FORM{'ism'}?1:0);
    $smicol = sprintf "%s",($FORM{'ism'}?"ISM_SMI":"SMILES");
    if ($FORM{'qtype'} eq "sub") {
      $sql .= "COLUMN SIM FORMAT 9.99;\n";
      $sim = sprintf "tanimoto(SMILES,'%s')",$FORM{'insmi'};
      $sql .=
        sprintf
"$select %s,$sim \"SIM\"\n\tFROM %s\n\tWHERE contains($smicol,$smi)=1",
            $dtypes,$tables;
      $order = " ORDER BY $sim DESC";
    } elsif ($FORM{'qtype'} eq "exact") {
      $sql .=
        sprintf
"$select %s\n\tFROM %s\n\tWHERE exact($smicol,$smi)=1",
            $dtypes,$tables;
    } elsif ($FORM{'qtype'} eq "sim") {
      $sql .= "COLUMN SIM FORMAT 9.99;\n";
      $sim = sprintf "tanimoto(SMILES,'%s')",$FORM{'insmi'};
      $sql .= sprintf
        "$select %s,$sim \"SIM\"\n\tFROM %s\n\tWHERE $sim>%.2f",
            $dtypes,$tables,$FORM{'simmin'};
      $order = " ORDER BY $sim DESC";
      if ($FORM{'ism'}) { push @ERRORS,"NOTE: Similarity is non-isomeric."; }
    } elsif ($FORM{'qtype'} eq "tver") {
      $sql .= "COLUMN SIM FORMAT 9.99;\n";
      $sim = sprintf "tversky(SMILES,'%s',%.2f,%.2f)",$FORM{'insmi'},
          $FORM{'alpha'},$FORM{'beta'};
      $sql .= sprintf "$select %s,$sim \"SIM\"\n\tFROM %s\n\tWHERE $sim>%.2f",
        $dtypes,$tables,$FORM{'simmin'};
      $order = " ORDER BY $sim DESC";
      if ($FORM{'ism'}) { push @ERRORS,"NOTE: Similarity is non-isomeric."; }
    } elsif ($FORM{'qtype'} eq "sma") {
      $sql .= sprintf "$select %s\n\tFROM %s\n\tWHERE matches($smicol,'%s')=1",
            $dtypes,$tables,$FORM{'insmi'};
    } else {
      push @ERRORS,"ERROR: No structural-querytype selected.";
    }

  #########################################################
  ### NAME query -- use VIEW WDI_ALLNAMES
  #########################################################
  } elsif ($FORM{'name'}) {
    $tables .= ',WDI_ALLNAMES';
    $dtypes =~ s/WDI_MAIN\.DXRN/WDI_ALLNAMES.DXRN/;
    $case = $FORM{'casesen'} ? "" : "UPPER";
    if ($FORM{'qtype'} eq "xnam") {
      $sql .= sprintf 
        "$select %s\n\tFROM %s\n\tWHERE %s(WDI_ALLNAMES.NAME)=%s('%s')",
        $dtypes,$tables,$case,$case,$FORM{'name'};
    } elsif ($FORM{'qtype'} eq "subnam") {
      $sql .= sprintf
        "$select %s\n\tFROM %s\n\tWHERE %s(WDI_ALLNAMES.NAME) LIKE %s('%%%s%%')",
        $dtypes,$tables,$case,$case,$FORM{'name'};
    } else {
      push @ERRORS,"ERROR: No NAME-querytype selected.";
    }

  #########################################################
  ### DXRN query
  #########################################################
  } elsif ($FORM{'dxrn'}) {
    $sql .= sprintf "$select %s\n\tFROM %s\n\tWHERE WDI_MAIN.DXRN=UPPER('%s')",
          $dtypes,$tables,$FORM{'dxrn'};

  #########################################################
  ### CAS query
  #########################################################
  } elsif ($FORM{'cas'}) {
    $sql .= sprintf "$select %s\n\tFROM %s\n\tWHERE WDI_CAS.CAS='%s'",
          $dtypes,$tables,$FORM{'cas'};

  #########################################################
  ### SQL input field query (replaces all other SQL)
  #########################################################
  } elsif ($FORM{'sql'}) {
    $sql = $FORM{'sql'};
    goto EOSQL;
  } else {
    push @ERRORS,"ERROR: Incomplete query spec.";
  }

  #########################################################
  ### Outer join using DXRN key if WDI and
  ### auxilliary tables are involved.
  ### For ALLNAMES search use WDI_ALLNAMES view for join.
  #########################################################
  if ($tables =~ /WDI_ALLNAMES/) {
    foreach (keys(%tables)) {
      $sql .= "\n\tAND WDI_ALLNAMES.DXRN=$_.DXRN(+)";
    }
  } elsif ($tables{'WDI_MAIN'}) {
    foreach (keys(%tables)) {
      next if ($_ eq 'WDI_MAIN');
      $sql .= "\n\tAND WDI_MAIN.DXRN=$_.DXRN(+)";
    }
  }

  #########################################################
  ### ORDER BY?
  #########################################################
  $sql .= $order;

  if ($FORM{'nhitsonly'}) {
    $sql =~ s/(SELECT[\s\S]*)$/SELECT COUNT(*) FROM (\n\t$1\n\t)/;
  }

EOSQL:
  printf TMP "%s;\nQUIT;\n",$sql;
  close(TMP);

  print "<PRE>\n";
  &SHOWSQL if ($FORM{'showsql'});
  &RUNSQL;
  print "</PRE></TD></TR></TABLE>\n";

  return (1); 
}

#############################################################################
### PING - Verify that Oracle and DayCart are responding.
#############################################################################
sub PING
{
  open(TMP,">${TMPFILE}.sql");
  print TMP "SELECT * FROM WDI_DBINFO;\n";
  print TMP "QUIT;\n";
  close(TMP);
  print "<HR><TABLE WIDTH=100% CELLPADDING=5><TR><TD BGCOLOR=WHITE><PRE>\n";
  &SHOWSQL;
  &RUNSQL;
  print "</PRE></TD></TR></TABLE>\n";
  return (1); 
}

#############################################################################
### DESCRIBE - 
#############################################################################
sub DESCRIBE
{
  open(TMP,">${TMPFILE}.sql");
  print TMP "SELECT * FROM WDI_DBINFO;\n";
  foreach (@TABLES,"WDI_ALLNAMES") {
    print TMP "DESCRIBE $_;\nSELECT COUNT(*) \"ROWS IN $_\" FROM $_;\n";
  }
  print TMP "QUIT;\n";
  close(TMP);
  print "<HR><TABLE WIDTH=100% CELLPADDING=5><TR><TD BGCOLOR=WHITE><PRE>\n";
  &SHOWSQL if ($FORM{'showsql'});
  &RUNSQL;
  print "</PRE></TD></TR></TABLE>\n";
  return (1); 
}

#############################################################################
### SHOWSQL - 
#############################################################################
sub SHOWSQL
{
  print "<FONT COLOR=RED>\n";
  system ("cat ${TMPFILE}.sql");
  print "</FONT>\n";
}

#############################################################################
### RUNSQL - 
#############################################################################
sub RUNSQL
{
  my $i;

  open(OUT,
    "$SQLPLUS $FORM{'user'}/$FORM{'pwd'}\@$FORM{'svr'} \@${TMPFILE}.sql |");
  for ($i=0; <OUT>; ++$i) {
    print "$_";
    if ($i>$FORM{'maxlines'}) {
      push @ERRORS, "NOTE: Output truncated at $FORM{'maxlines'} lines.";
      last;
    }
  }
  close(OUT);
}

###########################################################################
### HELP
###########################################################################
sub HELP
{
  print <<__EOF__;
<HR>
<H3>$PROG Help</H3>

This CGI provides basic access to WDI (DayCart/Oracle
version) via SQLPlus.  $PROG is a perl script using
no special purpose packages.

<P>

If a SMILES/SMARTS is entered, the other text fields below will
be ignored.  Likewise, if a Name is entered, the DXRN and
CAS fields below will be ignored, etc.  To input and run
SQL code, erase other inputs.

<P>

Selected datatypes will be displayed.  If none, DXRN will
be auto-selected.  The "Show SQL" feature both shows SQL
being run, and also activates the input SQL text field.
The "Show duplicates" field disables the DISTINCT modifier,
so that duplicate rows may be output.

<P>

DayCart debug levels (logfile is /tmp/extproc.log on server):
<P>
  0 - No logging at all.<BR>
  1 - Error messages only (the default value).<BR>
  5 - Warnings and errors.<BR>
  9 - Notes, warnings, and errors.<BR>


<P>
Author: <A HREF="mailto:jj\@daylight.com">Jeremy Yang</A>
<P>
__EOF__

  print "<HR>WDI Datatypes:<P>\n<PRE>\n";
  foreach (sort keys(%DTYPES)) {
    printf "\t%s\t%s\n", $_,$DTYPES{$_}->{head};
  }
  print "</PRE>\n";
}
