#!/usr/local/bin/perl
#############################################################################
### wdi2sql.pl - convert TDTs to sql INSERTs from mol2smi and dtypefix.pl,
### before merging by Thor, so there is one DXRN per TDT.
#############################################################################
### Some fields are one per TDT, others may be multiple.  We must know
### a priori to design and populate our tables.
#############################################################################
###  15 Aug 2001
#############################################################################

require DU_LIB;

%tree = ( );

$COMMITFREQ = 100; ### TDTs/commit

########################################################################
### WDI (main) table dtypes (TDT tags)
########################################################################
@TAGS = ('$DXRN','$SMI','ISM','SSK','DPN','PT',
    'AMW','2D_1','2D_2','DMF','DYQ','MA','JOUR_1','JOUR_2',
    'JOUR_3','SD','AE','PW','IA','CI','IU','REF','CONF_1','CONF_2','CONF_3',
    'CONF_4');

@MULTI = ('TN','COMB_PREP','USAN','APN','CAS','NAM','DRN','INN');

print "SET DEFINE OFF;  --- Prevent char substitution.\n\n";

$tdtcount = 0;

DITEM:
while ($ditem = <STDIN>) {
  chop($ditem);
  if ($ditem =~ /^\|$/) {
    ++$tdtcount;
    @NAM = &SORTUNIQ(@NAM);
    &PRINT_SQL();
    undef %tree;
    foreach (@MULTI) { undef @$_; }
    next;
  }
  @fields = &DU_DITEM2FIELDS($ditem);
  foreach (@fields) { s/'/''/g; }
  ($tag = $ditem) =~ s/^([^<]+)<(.*)$/$1/;

  ##################################################
  ### Handle multiply-occuring datatypes
  ##################################################
  foreach (@MULTI) { 
    if ($tag eq $_) { push @$_,join("\000",@fields); next DITEM; }
  }

###  ##################################################
###  ### Handle $NAM (as NAMs)
###  ##################################################
###  if ($tag eq '$NAM') {
###    push @NAM,@fields;
###    next DITEM;
###  }

  if ($#fields == 0) {
    $tree{$tag} = $fields[0];
  } else {
    $i=1;
    foreach $field (@fields) {
      $key = $tag."_".$i;
      $tree{$key} = $field;
      ++$i;
    }
  }
}

print "COMMIT;\n/***  $tdtcount TDTs  ***/\n\nQUIT;\n";

#############################################################################
##  PRINT_SQL - print sql INSERT statements for TDT
#############################################################################
sub PRINT_SQL
{
  if (!($tdtcount%$COMMITFREQ)) {
    print "COMMIT;\n/***  $tdtcount TDTs  ***/\n\n";
  }

  if (!exists($tree{'$DXRN'})) {
    printf STDERR "ERROR: no DXRN: %s\n", %tree;
    return;
  }
  $dxrn = $tree{'$DXRN'};

  print "INSERT INTO WDI_MAIN VALUES (";

  $first=1;
  foreach $tag (@TAGS) {
    if (!$tree{$tag}) {
      printf "%s\n\tNULL",($first?"":",");
    } elsif (length($tree{$tag})>4000) {
      if ($tag eq "2D_1") {   ### (renamed) D2D_1 is CLOB datatype
        printf "%s\n\t'%s'", ($first?"":","),
               join("' \|\|\n\t'", &SLICE($tree{$tag},2000));

      } else {
        printf STDERR "ERROR: Datum ($tag) len>4000 truncated: \"%s\".\n",
          $tree{$tag};
        substr($tree{$tag},3975) = " ---TRUNCATED BY DCIS---|";
        printf "%s\n\t'%s'", ($first?"":","),
               join("' \|\|\n\t'", &SLICE($tree{$tag},2000));
      }

    } elsif (length($tree{$tag})>2499) {
        printf "%s\n\t'%s'", ($first?"":","),
               join("' \|\|\n\t'", &SLICE($tree{$tag},2000));
    } else {
      printf "%s\n\t'%s'",($first?"":","),$tree{$tag};
    }
    $first=0;
  }
  print " );\n\n";

  #############################################################################
  ###  Now the (possibly) multiple datatypes for insertion into
  ###  auxilliary tables.
  ###    1-field: NAM,CAS,DRN
  ###    2-field: USAN,APN,INN
  ###    3-field: TN
  ###    4-field: COMB_PREP
  #############################################################################
  foreach $dt ('NAM','CAS','DRN') {  ### 1-field types
    foreach (@$dt) {
      printf "INSERT INTO WDI_$dt VALUES (\n\t'%s',\n\t'%s');\n\n",$dxrn,$_;
    }
  }
  
  foreach $dt ('APN','USAN','INN') {  ### 2-field types
    foreach (@$dt) {
      ($a,$b) = split(/\000/);
      printf "INSERT INTO WDI_$dt VALUES (\n\t'%s',\n\t'%s',\n",$dxrn,$a;
      if ($b) { printf "\t'%s'\n",$b; } else { print ",NULL\n"; }
      printf "\t);\n\n",
    }
  }
  
  foreach $dt ('TN') {  ### 3-field types
    foreach (@$dt) {
      ($a,$b,$c) = split(/\000/);
      printf "INSERT INTO WDI_$dt VALUES (\n\t'%s',\n\t'%s',\n",$dxrn,$a;
      if ($b) { printf "\t'%s',\n",$b; } else { print "NULL,\n"; }
      if ($c) { printf "\t'%s'\n",$c; } else { print ",NULL\n"; }
      printf "\t);\n\n",
    }
  }
  foreach $dt ('COMB_PREP') {  ### 4-field types
    foreach (@$dt) {
      ($a,$b,$c,$d) = split(/\000/);
      printf "INSERT INTO WDI_$dt VALUES (\n\t'%s',\n\t'%s',\n",$dxrn,$a;
      if ($b) { printf "\t'%s',\n",$b; } else { print "\tNULL,\n\t"; }
      if ($c) { printf "\t'%s',\n",$c; } else { print "\tNULL,\n\t"; }
      if ($d) { printf "\t'%s'\n",$d; }  else { print ",\tNULL\n"; }
      printf "\t);\n\n",
    }
  }
  return;
}

#############################################################################
### SLICE - divide a string into N-char substrings, return list
#############################################################################
sub SLICE
{
  my $str = shift;
  my $n = shift;
  my $i = 0;
  my @list;
  my $len = length($str);

  while ($i<$len) {
    push @list,substr($str,$i,$n);
    $i += $n;
  }
  return @list;
}

#############################################################################
### SORTUNIQ
#############################################################################
sub SORTUNIQ
{
  my @list = sort(@_);
  my @new;
  my $last;

  foreach (@list) {
    if ($_ ne $last) { push @new,$_; }
    $last = $_;
  }
  return @new;
}
